@@ -13,12 +13,15 @@ from .utils import (
     ArrayLike,
     LatticeError,
     ConfigurationError,
+    SiteOccupiedError,
+    NoConnectionsError,
+    NotAnalyzedError,
+    NotBuiltError,
     min_dtype,
     chain,
     frmt_num,
     frmt_bytes,
     frmt_time,
-    Timer
 )
 
 from .spatial import (
@@ -30,18 +33,14 @@ from .spatial import (
     cell_size,
     cell_volume,
     compute_vectors,
-    compute_neighbors,
     VoronoiTree,
     WignerSeitzCell
 )
 
-from .unitcell import (
-    Atom,
-    UnitCell
-)
-
+from .shape import AbstractShape, Shape, Circle, Donut, ConvexHull
 from .data import LatticeData
 from .disptools import DispersionPath
+from .unitcell import Atom
 from .lattice import Lattice
 
 from ._version import get_versions
