@@ -22,27 +22,28 @@ from .plotting import draw_points, draw_vectors, draw_lines, draw_surfaces
 
 __all__ = [
     "distance", "distances", "interweave", "vindices", "vrange", "cell_size",
-    "cell_volume", "compute_vectors", "compute_neighbors", "KDTree", "VoronoiTree",
+    "cell_volume", "compute_vectors", "KDTree", "VoronoiTree",
     "WignerSeitzCell", "rx", "ry", "rz", "rotate2d", "rotate3d",
-    "build_periodic_translation_vector"
+    "build_periodic_translation_vector", "periodic_translation_vectors"
 ]
 
 
-def distance(r1: ArrayLike, r2: ArrayLike, decimals: int = None) -> float:
-    """ Calculates the euclidian distance bewteen two points.
+def distance(r1: np.ndarray, r2: np.ndarray, decimals: int = None) -> float:
+    """Calculates the euclidian distance bewteen two points.
 
     Parameters
     ----------
-    r1: array_like
+    r1 : (D, ) np.ndarray
         First input point.
-    r2: array_like
+    r2 : (D, ) np.ndarray
         Second input point of matching size.
     decimals: int, optional
-        Optional decimals to round distance to.
+        Decimals for rounding the output.
 
     Returns
     -------
     distance: float
+        The distance between the input points.
     """
     dist = math.sqrt(np.sum(np.square(r1 - r2)))
     if decimals is not None:
@@ -51,22 +52,22 @@ def distance(r1: ArrayLike, r2: ArrayLike, decimals: int = None) -> float:
 
 
 def distances(r1: ArrayLike, r2: ArrayLike, decimals: int = None) -> np.ndarray:
-    """ Calculates the euclidian distance between multiple points.
+    """Calculates the euclidian distance between multiple points.
 
     Parameters
     ----------
-    r1: array_like
+    r1 : (N, D) array_like
         First input point.
-    r2: array_like
+    r2 : (N, D) array_like
         Second input point of matching size.
     decimals: int, optional
         Optional decimals to round distance to.
 
     Returns
     -------
-    distance: np.ndarray
+    distances : (N, ) np.ndarray
+        The distances for each pair of input points.
     """
-
     r1 = np.atleast_2d(r1)
     r2 = np.atleast_2d(r2)
     dist = np.sqrt(np.sum(np.square(r1 - r2), axis=1))
@@ -224,13 +225,16 @@ def cell_volume(vectors: ArrayLike) -> float:
     r""" Computes the volume of the unit cell defined by the primitive vectors.
 
     The volume of the unit-cell in two and three dimensions is defined by
+
     .. math::
         V_{2d} = \abs{a_1 \cross a_2}, \quad V_{3d} = a_1 \cdot \abs{a_2 \cross a_3}
 
     For higher dimensions the volume is computed using the determinant:
+
     .. math::
         V_{d} = \sqrt{\det{A A^T}}
-    where .math:`A` is the array of vectors.
+
+    where :math:`A` is the array of vectors.
 
     Parameters
     ----------
@@ -240,6 +244,7 @@ def cell_volume(vectors: ArrayLike) -> float:
     Returns
     -------
     vol: float
+        The volume of the unit cell.
     """
     dim = len(vectors)
     if dim == 1:
@@ -254,14 +259,6 @@ def cell_volume(vectors: ArrayLike) -> float:
     return abs(v)
 
 
-def build_periodic_translation_vector(indices, axs):
-    limits = np.array([np.min(indices, axis=0), np.max(indices, axis=0)])
-    nvec = np.zeros(indices.shape[1] - 1, dtype=np.int64)
-    for ax in np.atleast_1d(axs):
-        nvec[ax] = np.floor(limits[1][ax]) + 1
-    return nvec
-
-
 def compute_vectors(a: float, b: float = None, c: float = None,
                     alpha: float = None, beta: float = None,
                     gamma: float = None,
@@ -298,6 +295,7 @@ def compute_vectors(a: float, b: float = None, c: float = None,
     return vectors
 
 
+# noinspection PyUnresolvedReferences
 class KDTree(scipy.spatial.cKDTree):
     """Simple wrapper of scipy's cKTree with global query settings."""
 
@@ -308,6 +306,9 @@ class KDTree(scipy.spatial.cKDTree):
         self.p = p
         self.eps = eps
 
+    def __repr__(self):
+        return f"{self.__class__.__name__}(k: {self.k}, p: {self.p}, eps: {self.eps})"
+
     def query_ball_point(self, x, r, *_):
         return super().query_ball_point(x, r, self.p, self.eps)
 
@@ -348,13 +349,30 @@ class KDTree(scipy.spatial.cKDTree):
         return neighbors, dists
 
 
-def compute_neighbors(positions, k=20, max_dist=np.inf, num_jobs=1, decimals=None,
-                      eps=0., include_zero=False, compact=True, x=None):
-    # Build tree and query neighbors
-    x = positions if x is None else x
-    tree = KDTree(positions, k=k, max_dist=max_dist, eps=eps)
-    dists, neighbors = tree.query(x, num_jobs, decimals, include_zero, compact)
-    return neighbors, dists
+def build_periodic_translation_vector(indices, axs):
+    limits = np.array([np.min(indices, axis=0), np.max(indices, axis=0)])
+    idx_size = (limits[1] - limits[0])[:-1]
+    nvec = np.zeros_like(idx_size, dtype=np.int64)
+    for ax in np.atleast_1d(axs):
+        nvec[ax] = np.floor(idx_size[ax]) + 1
+    return nvec
+
+
+def periodic_translation_vectors(indices, axs):
+    # One axis: No combinations needed
+    if isinstance(axs, int) or len(axs) == 1:
+        return [(axs, build_periodic_translation_vector(indices, axs))]
+    # Add all combinations of the periodic axis
+    items = list()
+    for ax in itertools.combinations_with_replacement(axs, r=2):
+        nvec = build_periodic_translation_vector(indices, ax)
+        items.append((ax, nvec))
+        # Use +/- for every axis exept the first one to ensure all corners are hit
+        if not np.all(np.array(ax) == axs[0]):
+            nvec2 = np.copy(nvec)
+            nvec2[1:] *= -1
+            items.append((ax, nvec2))
+    return items
 
 
 class VoronoiTree:
@@ -387,8 +405,9 @@ class VoronoiTree:
     def query(self, x, k=1, eps=0):
         return self.tree.query(x, k, eps)  # noqa
 
-    def draw(self, ax=None, color="C0", size=3, lw=1, alpha=0.15, point_color="k",
-             point_size=3, draw_data=True, points=True, draw=True, fill=True):
+    def draw(self, ax=None, color="C0", size=3, lw=1, alpha=0.15,
+             point_color="k", point_size=3, draw_data=True,
+             points=True, draw=True, fill=True):  # pragma: no cover
 
         if ax is None:
             fig = plt.figure()
@@ -479,7 +498,7 @@ class WignerSeitzCell(VoronoiTree):
         return grid
 
     def symmetry_points(self):
-        origin = np.zeros((1,))
+        origin = np.zeros(self.dim)
         corners = self.vertices.copy()
         face_centers = None
         if self.dim == 1:
@@ -524,7 +543,7 @@ def rz(theta: float) -> np.ndarray:
     return np.array([[cos, -sin, 0], [sin, cos, 0], [0, 0, 1]])
 
 
-def rot(thetax: float = 0., thetay: float = 0., thetaz: float = 0.) -> np.ndarray:
+def rot(thetax: float = 0., thetay: float = 0., thetaz: float = 0.):  # pragma: no cover
     """General rotation matrix"""
     r = np.eye(3)
     if thetaz:
@@ -536,14 +555,14 @@ def rot(thetax: float = 0., thetay: float = 0., thetaz: float = 0.) -> np.ndarra
     return r
 
 
-def rotate2d(a, theta, degree=True):
+def rotate2d(a, theta, degree=True):  # pragma: no cover
     """Applies the z-rotation matrix to a 2D point"""
     if degree:
         theta = np.deg2rad(theta)
     return np.dot(a, rz(theta)[:2, :2])
 
 
-def rotate3d(a, thetax=0., thetay=0., thetaz=0., degree=True):
+def rotate3d(a, thetax=0., thetay=0., thetaz=0., degree=True):  # pragma: no cover
     """Applies the general rotation matrix to a 3D point"""
     if degree:
         thetax = np.deg2rad(thetax)
